package puppet.ambition;

import info.frantzen.testing.ssmsimulator.ssm.InteractionVariable;
import info.frantzen.testing.ssmsimulator.types.ComplexType;
import info.frantzen.testing.ssmsimulator.types.ComplexTypeInstance;
import info.frantzen.testing.ssmsimulator.types.ST_Boolean;
import info.frantzen.testing.ssmsimulator.types.ST_PosInt;
import info.frantzen.testing.ssmsimulator.types.ST_PosIntInstance;
import info.frantzen.testing.ssmsimulator.types.ST_PseudoPosDouble;
import info.frantzen.testing.ssmsimulator.types.ST_String;
import info.frantzen.testing.ssmsimulator.types.Type;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.antlr.stringtemplate.StringTemplate;

public class MethodCodeProcessor {

    private String name;
    
    private String methodType;
    
    private AmbitionVariableWrapper ambitionVarWrapper;
    
    private PatternsReader patterns;
    
// Are the qualificator such as "private", "public" and so on    
    private String methodPrefix;

    private ArrayList<InteractionVariable> parametersList;

    
    public MethodCodeProcessor(String methodName, String methodPrefix) {
        this.methodPrefix = methodPrefix;
        this.methodType = "void";
        parametersList = new ArrayList<InteractionVariable>();
        ambitionVarWrapper = new AmbitionVariableWrapper();
        patterns = new PatternsReader();
        name = methodName;
    }

    public MethodCodeProcessor(String methodName) {
        this(methodName,"public");
    }


    private String getAmbitionInvocationCode (ArrayList<InteractionVariable> parametersList){
        String generatedCode="";
        String instanceParameters = "";
        InteractionVariable var;

        generatedCode = patterns.getMessageSelectionCode(this.isMethodVoid());
        
        for (int i=0; i < parametersList.size(); i++ ){
           var = parametersList.get(i);

/*         Format valid for ST_PosInt, ST_Boolean, ST_PseudoPosDouble, ST_String
*/
           generatedCode += "\n\n /* Generated Parameter "+ i + "\n */ \n";
           generatedCode += "$ambitionPrefix$var = (info.frantzen.testing.ssmsimulator.ssm.InteractionVariable) $ambitionPrefix$it.next();";

           if (var.getType() instanceof ComplexType ){
//               generatedCode += "info.frantzen.testing.ssmsimulator.types.TypeInstance "+ var.getName() + "Instance = new info.frantzen.testing.ssmsimulator.types." + var.getType().getTypeIdentifier() + "Instance("+var.getName()+"); \n";
               
/*               java.util.ArrayList<info.frantzen.testing.ssmsimulator.types.Type> $ambitionPrefix$typeList = $ambitionPrefix$ssm.getTypes();
               java.util.Iterator $ambitionPrefix$iter = $ambitionPrefix$typeList.iterator();
               info.frantzen.testing.ssmsimulator.types.ComplexType $ambitionPrefix$parameter;
               while ($ambitionPrefix$iter.hasNext()){
                   info.frantzen.testing.ssmsimulator.types.Type $ambitionPrefix$typeItem = $ambitionPrefix$iter.next();
                   if ($ambitionPrefix$typeItem.getName().equals(" CHANGE IT --> var.getType().getName() <--" )){
                       $ambitionPrefix$parameter = (info.frantzen.testing.ssmsimulator.types.ComplexType) $ambitionPrefix$typeItem;
                       break;
                   }
               }

               Object [][] $ambitionPrefix$fields = $ambitionPrefix$parameter.getFields();
               Object [] $ambitionPrefix$parameterValues = new Object [$ambitionPrefix$fields.length];
               for (int i=0; i< $ambitionPrefix$parameterValues.length; i++){
                   if (((info.frantzen.testing.ssmsimulator.types.Type) $ambitionPrefix$fields[i][1]) instanceof info.frantzen.testing.ssmsimulator.types.ST_PosInt)
                       $ambitionPrefix$parameterValues[i] = new info.frantzen.testing.ssmsimulator.types.ST_PosIntInstance(" CHANGE IT --> var.getName() <--".get$ambitionPrefix$fields[i][0])                                     
               }
*/
               
               Object [][] fields = ((ComplexType) var.getType()).getFields();
               generatedCode += " Object [] $ambitionPrefix$parameterValues = new Object [" + fields.length + "]; ";                   
               for (int index=0; index < fields.length; index++){
                   generatedCode += "$ambitionPrefix$parameterValues["+index+"] = new info.frantzen.testing.ssmsimulator.types."+ fields[index][1] +"Instance("+var.getName()+"."+fields[index][0]+"());";          
               }
               generatedCode += "info.frantzen.testing.ssmsimulator.types.TypeInstance "+ var.getName() + "Instance = new info.frantzen.testing.ssmsimulator.types." + var.getType().getTypeIdentifier() + "Instance( $ambitionPrefix$parameterValues ); \n";
               
           }
           else
               generatedCode += "info.frantzen.testing.ssmsimulator.types.TypeInstance "+ var.getName() + "Instance = new info.frantzen.testing.ssmsimulator.types." + var.getType().getTypeIdentifier() + "Instance("+var.getName()+"); \n";

           generatedCode += "$ambitionPrefix$valuation.addSingleValuation($ambitionPrefix$var.getName(), "+var.getName()+"Instance); \n";
        }   
        
//        generatedCode += "jambition.Fuppet $ambitionPrefix$functionalEmulator = new jambition.Fuppet(valuation); \n";
        if (isMethodVoid())        
            generatedCode += patterns.getSimulatorInvocationCode();
        else {
            TypeConverter converter = new TypeConverter ();
            if (!converter.isAnAmbitionComplexType(this.methodType))
                generatedCode += patterns.getSimulatorInvocationCode(this.methodType);
            else{
                generatedCode += this.castComplexType(this.methodType);
            }
        }
        
        StringTemplate templateParser = new StringTemplate(generatedCode);
        templateParser.setAttribute("ambitionPrefix", ambitionVarWrapper.getAmbitionPrefix());        
        templateParser.setAttribute("operationName", this.name);        
        generatedCode = templateParser.toString();
        
//        return "/* Code Generated for Integration with Ambition \n */ \n" + generatedCode +" \n return \"gulyx\";";
        return "/* Code Generated for Integration with Ambition \n */ \n" + generatedCode;
    }

    
    private String castComplexType(String inputType){
        String castingString = patterns.getSimulatorInvocationCodeComplexTypes();
        Class<?> genericClass;
        Constructor[] constructorList;
        Class[] constructorArgumentsTypes;
        
        try {

            genericClass = Class.forName(inputType);
            constructorList = genericClass.getConstructors();
            
//            Looks for the Constructor with the max numeber of parameters
            constructorArgumentsTypes = new Class[]{};
            for ( int index = 0; index < constructorList.length ; index++){
                if (constructorArgumentsTypes.length < constructorList[index].getParameterTypes().length ){
                    constructorArgumentsTypes = constructorList[index].getParameterTypes();
System.out.println(genericClass.getName() + " Changed at " + constructorArgumentsTypes.length +" -----------------------------");                    
                }
            }    
            
            // Here the code that generate the casting statements.
            castingString += " String [] $ambitionPrefix$arrayRepresentation = $ambitionPrefix$returnInstance.toString().split(\",\");";
            castingString += " $ambitionPrefix$"+ patterns.getReturnValueVarName() +" = new " + inputType + "(";
            for (int index=0; index < constructorArgumentsTypes.length-1; index++){
    System.out.println("+++++++++++ "+ index +") Arg: arg" + index + " Type: "+ constructorArgumentsTypes[index].getName() + "++++++++++");
                String castedStatement = patterns.castStatement("$ambitionPrefix$arrayRepresentation["+index+"]", constructorArgumentsTypes[index].getName());
//                castingString += "$ambitionPrefix$arrayRepresentation["+index+"],";
                castingString += castedStatement+",";
            }
// I need to do that since I do not have to insert the last ","            
            if (constructorArgumentsTypes.length-1 > 0){
              String lastIndex = Integer.toString(constructorArgumentsTypes.length-1);
              castingString += patterns.castStatement("$ambitionPrefix$arrayRepresentation["+lastIndex+"]", constructorArgumentsTypes[constructorArgumentsTypes.length-1].getName());
//                castingString += "$ambitionPrefix$arrayRepresentation["+lastIndex+"]";    
            }
            castingString += ");";
            castingString += "\n /* Now send the returnValue back to the calling service. That's it.*/ \n";

        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }                      
        return castingString;
    }

    private ArrayList<InteractionVariable> extractParametersList (String code){
        ArrayList<InteractionVariable> methodParametersList = new ArrayList<InteractionVariable>();
        
        String methodRegExpr = this.name+"\\s*[(][^)]*[)]"; 
        
        Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
        Matcher methodDefinitionMatcher = methodPattern.matcher(code); 
         
        String methodDefinitionString;
        String methodSignatureString;
        String[] tempBuffer;
        String[] methodParameter;
        
        TypeConverter converter = new TypeConverter();

        while (methodDefinitionMatcher.find()){
            methodDefinitionString = methodDefinitionMatcher.group();
            tempBuffer = methodDefinitionString.split("[()]");
            if (tempBuffer.length == 2){
                methodSignatureString =  tempBuffer[1];
                tempBuffer = methodSignatureString.split("\\s*[,]\\s*");
                for (int i=0; i<tempBuffer.length; i++){
                    methodParameter = tempBuffer[i].split("\\s+");
// Due the upgrade of the SSMSimulator.jar
//                  InteractionVariable var = new InteractionVariable(methodParameter[1],"",converter.getConvertedAmbitionType(methodParameter[0]));
                  InteractionVariable var = new InteractionVariable(methodParameter[1],converter.getConvertedAmbitionType(methodParameter[0]));
                    methodParametersList.add(var);
                }
            }
        }
        
        return methodParametersList;
    }
    
    
    private String getAmbitionReturnCode(){
        String generatedCode = patterns.getReturnCode();

        StringTemplate templateParser = new StringTemplate(generatedCode);
        templateParser.setAttribute("ambitionPrefix", ambitionVarWrapper.getAmbitionPrefix());        
        generatedCode = templateParser.toString();

        return generatedCode;
    }
    
    private boolean isMethodVoid (){
       return this.methodType.equals("void");   
    }
    
    private String getReturnRegExpr(){        
        if (isMethodVoid())
            return "";
        return "return\\s+[^;]+[;]";   
    }
    
    private String getMethodRegExpr(){
        if (isMethodVoid())
            return this.name+"\\s*[(][^{]*[{]";
        return this.name+"\\s*[(][^{]*[{][\\p{Punct}\\p{Alnum}\\p{Space}]*?"+getReturnRegExpr()+"\\p{Space}*[}]";    
    }
    
    public String parseCode(String code){
        String tempBuffer;
        String localSourceCodeBuffer = code;
                
        if (this.name != null){
            
            this.methodType = extractMethodType(code);
            
            parametersList = extractParametersList(code);
//            parametersList = extractParametersList(this.name);

//            String returnRegExpr = "return\\s+[^;]+[;]";
//            String methodRegExpr = this.name+"\\s*[(][^{]*[{][\\p{Punct}\\p{Alnum}\\p{Space}]*?"+returnRegExpr+"\\p{Space}*[}]"; 

            String returnRegExpr = getReturnRegExpr();
            String methodRegExpr = getMethodRegExpr(); 

            Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
            Matcher methodDefinitionMatcher = methodPattern.matcher(code);

            while (methodDefinitionMatcher.find()){
                
                String ambitionInvoString = this.getAmbitionInvocationCode(parametersList);
                String ambitionRetunrString = this.getAmbitionReturnCode();
                
                String methodDefinitionString = methodDefinitionMatcher.group();
                String ambitionTimeStampString = this.patterns.getCurrentTimeStampStatements();

                ambitionInvoString = ambitionTimeStampString + " \n " + ambitionInvoString;
                ambitionInvoString = patterns.castExceptions(ambitionInvoString, "java.rmi.RemoteException");

                ambitionInvoString = patterns.getCommonVariableCode(this.methodType) + ambitionInvoString; 
                
                if (!methodDefinitionString.contains(methodPrefix) ){
// Add the statements to get the timeStamp at invocation time                    
                    tempBuffer = methodDefinitionString.replaceFirst("\\{", "\\{" + ambitionInvoString); 
//                    tempBuffer = methodDefinitionString.replaceFirst("\\{", "\\{" + ambitionTimeStampString); 

                    if (!isMethodVoid())
                        tempBuffer = tempBuffer.replaceAll(returnRegExpr, ambitionRetunrString);
//                    tempBuffer = tempBuffer.replaceAll(returnRegExpr, ambitionInvoString);
                    
                    int start = methodDefinitionMatcher.start();
                    int offset = methodDefinitionMatcher.end();
                                        
                    localSourceCodeBuffer = this.exchangePortion(code, start, offset, tempBuffer);

                    localSourceCodeBuffer = localSourceCodeBuffer.replaceAll("/\\*\\s*LocalComputationEffort\\s*\\*/", patterns.getElapsedTimeStatement());
                
                }
            }
        }
        return localSourceCodeBuffer;
    }

    private String extractMethodType(String code) {
        String methodRegExpr = this.methodPrefix+"\\p{Space}+\\p{Alpha}[\\p{Alnum}_.]*\\p{Space}+"+this.name+"\\s*[(]"; 

        Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
        Matcher methodDefinitionMatcher = methodPattern.matcher(code);

        String returnedMethodType = "void";
        
// If there is no overloading as we are supposing, it should be executed just once.        
        while (methodDefinitionMatcher.find()){
            String methodDefinitionString = methodDefinitionMatcher.group();
            String [] methodDefinitionParts = methodDefinitionString.split("\\s");            
            returnedMethodType = methodDefinitionParts[1];
        }
        
        return returnedMethodType;
    }

    private String exchangePortion(String originalString, int beginRegion, int endRegion, String newString){

        int originalStringLength =  originalString.length();
        String exchangedString = originalString.substring(0,beginRegion)+newString+originalString.substring(endRegion, originalStringLength);            
        return exchangedString;
    }
    
    
    public String getName() {
        return name;
    }

    public ArrayList<InteractionVariable> getParametersList() {
        return parametersList;
    }

    public String getMethodType() {
        return methodType;
    }

}
