/*
 * Operation.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 10. Dezember 2006, 19:26
 *
 */

package info.frantzen.testing.ssmsimulator.ssm;

/**
 * This is exactly the same notion as a WSDL operation.
 * 
 * <B>To be checked:</B> an Operation may appear in several port types, so maybe we have
 * also relate an operation to a Port?
 * @author Lars Frantzen
 * @version 0.2
 */
public class Operation {
    
    private String name;
    
    private OperationKind kind;
    
    /**
     * Create a new Operation instance.
     * @param name the name of the Operation
     * @param kind the kind of the Operation
     */
    public Operation(String name, OperationKind kind) {
        this.name = name;
        this.kind = kind;
    }
    
    /**
     * Getter for the name attribute.
     * @return the name of the Operation
     */
    public String getName() {
        return name;
    }
       
    /**
     * Getter for the kind attribute.
     * @return the kind of the Operation
     */
    public OperationKind getKind() {
        return kind;
    }
    
    /**
     * Checks the consistency, i.e.<br>
     * <CODE>(name != "") && (kind != null)</CODE>
     * @return true if the consistency is ok, false otherwise
     */
    public boolean checkConsistency() {
        return ((name != "") && (kind != null));
    }
    
    /**
     * A string representation of the operation.
     * @return a string representation of the operation
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(name + '(');
        if (kind == OperationKind.NOTIFICATION)
            sb.append("NOTIFICATON");
        if (kind == OperationKind.ONEWAY)
            sb.append("ONEWAY");
        if (kind == OperationKind.REQUESTRESPONSE)
            sb.append("REQUESTRESPONSE");
        if (kind == OperationKind.SOLICITRESPONSE)
            sb.append("SOLICITRESPONSE");
        sb.append(')');
        return sb.toString();
    }
    
    public boolean equals(Operation op) {
        return (name.equals(op.getName()) && kind == op.getKind());
    }
    
}
