/*
 * Valuation.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 * Created on January 10, 2007, 4:02 PM
 *
 */

package info.frantzen.testing.ssmsimulator.ssm;

import java.util.Enumeration;
import java.util.Hashtable;
import info.frantzen.testing.ssmsimulator.types.ComplexTypeInstance;
import info.frantzen.testing.ssmsimulator.types.TypeInstance;

/**
 * A valuation assigns type instances to variable names.
 * @author Lars Frantzen
 * @version 0.1
 */
public class Valuation {
    
    
    private Hashtable<String, TypeInstance> valuation;
    
    
    /** Creates a new instance of Valuation */
    public Valuation() {
        valuation = new Hashtable<String, TypeInstance>();
    }
    
    /**
     * Adds a single (variable name) -> (type instance) mapping to the Valuation.
     * @param name the name of the variable
     * @param instance the type instance of the variable
     */
    public void addSingleValuation(String name, TypeInstance instance) {
        valuation.put(name, instance);
    }
    
    /**
     * Returns the type instance for a given variable name.
     * @param name the variable name
     * @return the type instance of the passed variable name
     */
    public  TypeInstance getSingleInstance(String name) {
        return valuation.get(name);
    }
    
    /**
     * Clones this Valuation.
     * @return the cloned Valuation
     */
    public Valuation clone() {
        Valuation clonedVal = new Valuation();
        
        for (Enumeration keys = valuation.keys() ; keys.hasMoreElements() ; ) {
            String key = (String) keys.nextElement();
            clonedVal.addSingleValuation(key, valuation.get(key).clone());
        }
        
        return clonedVal;
    }
    
    public Hashtable<String, TypeInstance> getValuationTable() {
        return valuation;
    }
    
    /**
     * A string representation of this Valuation.
     * @return a string representation of this Valuation.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        
        for (Enumeration keys = valuation.keys() ; keys.hasMoreElements() ; ) {
            String key = (String) keys.nextElement();
            sb.append(key).append("(");
            TypeInstance ti = valuation.get(key);
            sb.append(ti);
            sb.append(")");
//            if (!(ti instanceof ComplexTypeInstance))
//                sb.append('\n');
            if (keys.hasMoreElements())
                sb.append(',');
        }
        
        return sb.toString();
    }
}
