/*
 * ComplexTypeInstance.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 14:56
 *
 */

package info.frantzen.testing.ssmsimulator.types;

/**
 * A complex type instance represents an instance of an {@link ComplexType complex type}, i.e.
 * a list of {@link TypeInstance type instances}.
 * @author Lars Frantzen
 * @version 0.1
 */
public class ComplexTypeInstance extends TypeInstance {
    
    /**
     * Creates a new instance of ComplexTypeInstance
     * @param fieldvalues the list of type instances
     */
    public ComplexTypeInstance(Object[] fieldvalues) {
        super(fieldvalues);
    }
    
    /**
     * A string representation of a complex type instance.
     * @return a string representation of a complex type instance
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        Object[] val = (Object[]) getValue();
        for (int i=0; i<val.length; i++) {
            sb.append(((TypeInstance) val[i]).toString());
            if (i < val.length-1)
                sb.append(",");
        }
        return sb.toString();
    }
    
//    public String toString() {
//        StringBuffer sb = new StringBuffer();
//        sb.append('\n');
//        Object[] val = (Object[]) getValue();
//        for (int i=0; i<val.length; i++) {
//            sb.append(i + ") ");
//            sb.append(((TypeInstance) val[i]).toString());
//            sb.append("\n");
//        }
//        return sb.toString();
//    }
    
    /**
     * Clones a complex type instance.
     * @return the cloned complex type instance
     */
    public ComplexTypeInstance clone() {
        Object[] orig = (Object[]) super.getValue();
        Object[] copy = orig.clone();
        
        for (int i = 0; i < copy.length; i++)
            copy[i] = ((TypeInstance) orig[i]).clone();
        
        return new ComplexTypeInstance(copy);
    }
}
